﻿using System;
using System.Linq;

using Twitchbot;
using Twitchbot.Core;
using Twitchbot.Util;

using CirclePrefect.Dotnet;

namespace Quotes.v2
{
    [ApiVersion(0, 1)]
    public class Plugin : TwitchBot
    {
        public override string Name => "Quotes v2";
        public override Version Version => new Version(1, 0);
        private DataStore DB;
        private const string Log = "Quotes_Log";
        public override void Initialize()
        {
            DB = new DataStore("quoteDB");
            for (int i = 0; i < DB.block.Length; i++)
            {
                if (string.IsNullOrEmpty(DB.block[i].Heading)) continue;
                if (DB.block[i].GetValue("active") == "false")
                    DB.block[i].active = false;
            }
            Bot.MessageReceivedEvent += Bot_MessageReceivedEvent;
        }

        private void Bot_MessageReceivedEvent(object sender, Bot.MessageReceivedArgs e)
        {
            if ((e.Mod.Value || e.BC.Value) && IsCommand(e.Message) && Operator(e.Message))
            {
                if (AddCommand(e.Message))
                {
                    string name = "";
                    if (!DB.BlockExists(name = Heading(e.Message)))
                    {
                        var block = DB.NewBlock(new[] { "quote", "active" }, name.ToLower().TrimEnd(' '));
                        block.WriteValue("quote", Quote(e.Message));
                        block.WriteValue("active", "true");
                        block.active = true;
                        Bot.Instance.SendMessage(string.Concat("Quote #", DB.block.Where(t => t.active).Count() + 1, " added"), e.Channel);
                        DB.WriteToFile();
                        return;
                    }
                }
                else if (RemoveCommand(e.Message))
                {
                    string name = "";
                    if (DB.BlockExists(name = HeadingSimple(e.Message)))
                    {
                        var block = DB.GetBlock(name);
                        if (block.GetValue("active") == "true")
                        {
                            block.WriteValue("active", "false");
                            block.active = false;
                            Bot.Instance.SendMessage(string.Concat("Quote #", QuoteIndex(block), " removed"), e.Channel);
                            DB.WriteToFile();
                        }
                        return;
                    }
                }
                else if (EditCommand(e.Message))
                {
                    string name = "";
                    if (DB.BlockExists(name = Heading(e.Message)))
                    {
                        var block = DB.GetBlock(name);
                        block.WriteValue("quote", Quote(e.Message));
                        block.WriteValue("active", "true");
                        Bot.Instance.SendMessage(string.Concat("Quote #", QuoteIndex(block), " modified"), e.Channel);
                        block.active = true;
                        DB.WriteToFile();
                        return;
                    }
                }
            }
            if (IsCommand(e.Message) && IsQuote(e.Message))
            {
                var block = DB.GetBlock(e.Message.Substring(1).ToLower().TrimEnd(' '));
                if (block.GetValue("active") == "true")
                {
                    Bot.Instance.SendMessage(block.GetValue("quote"), e.Channel);
                }
            }
        }

        private int QuoteIndex(Block block)
        {
            return DB.block.ToList().IndexOf(block) + 1;
        }
        private bool IsCommand(string value)
        {
            return value.Substring(0, 1) == "!" || value.Substring(0, 1) == "~";
        }
        private bool Operator(string value)
        {
            return value.Contains("@");
        }
        private bool AddCommand(string value)
        {
            return (value.Contains("add") || value.Contains("addquote")) && value.Count(t => t == '"') == 2;
        }
        private bool RemoveCommand(string value)
        {
            return value.Contains("remove");
        }
        private bool EditCommand(string value)
        {
            return value.Contains("edit") || value.Contains("editquote") || value.Contains("modify");
        }
        private string Heading(string value)
        {
            string name = value.Substring(value.IndexOf("@"));
            int start = name.IndexOf('"');
            int end = name.IndexOf(' ');
            if (start > end)
            {
                name = name.Substring(1, end);
            }
            return name.TrimEnd(' ');
        }
        private string HeadingSimple(string value)
        {
            string name = value.Substring(value.IndexOf("@"));
            return name.Substring(1);
        }
        private string Quote(string value)
        {
            string start = value.Substring(value.IndexOf('"') + 1);
            return start.Substring(0, start.LastIndexOf('"'));
        }
        private bool IsQuote(string value)
        {
            value = value.TrimStart(new[] { '~', '!' });
            for (int i = 0; i < DB.block.Length; i++)
            {
                if (string.IsNullOrEmpty(DB.block[i].Heading)) continue;
                string name = DB.block[i].Heading.TrimEnd(' ');
                if (name.StartsWith(value) && name.Length == value.Length)
                {
                    return true;
                }
            }
            return false;
        }

        public override void Dispose()
        {
            Bot.MessageReceivedEvent -= Bot_MessageReceivedEvent;
        }
    }
}
